@@ -7,8 +7,8 @@ import numpy as np
 import astropy.units as u
 import pytest
 
-from ..base import (BaseTaskBase, SetAttribute, TaskBase, PaddedTaskBase,
-                    Task)
+from baseband_tasks.base import (
+    BaseTaskBase, SetAttribute, TaskBase, PaddedTaskBase, Task)
 from .common import UseVDIFSample
 
 
@@ -146,6 +146,23 @@ class TestSetAttribute(UseVDIFSample):
         assert data.dtype == 'f2'
         assert np.all(data == expected.astype('f2'))
 
+    def test_metadata_propagation(self):
+        sa = SetAttribute(self.fh)
+        sa.meta['parrot'] = 'dead'
+        assert not hasattr(sa, 'frequency')
+        sa2 = SetAttribute(sa, frequency=300*u.MHz, sideband=1)
+        assert sa2.frequency is not None
+        assert sa2.sideband is not None
+        assert set(sa2.meta) == {'parrot', '__attributes__'}
+        assert sa2.meta['parrot'] == 'dead'
+        sa2.meta['parrot'] = 'goner'
+        assert sa2.meta['parrot'] == 'goner'
+        assert sa.meta['parrot'] == 'dead'
+
+    def test_fail_on_unknown_attribute(self):
+        with pytest.raises(TypeError):
+            SetAttribute(self.fh, freq=1.*u.MHz)
+
 
 class TestTaskBase(UseVDIFSample):
     def test_basetaskbase(self):
@@ -471,11 +488,12 @@ class TestTasks(UseVDIFSample):
 
 
 class TestPaddedTaskBase(UseVDIFSample):
+    @pytest.mark.parametrize('samples_per_frame', [None, 128])
     @pytest.mark.parametrize('n', [3, 1])
-    def test_basics(self, n):
+    def test_basics(self, n, samples_per_frame):
         fh = self.fh
-        sh = SquareHat(fh, n)
-        expected_size = ((fh.shape[0] - n + 1) // 4) * 4
+        sh = SquareHat(fh, n, samples_per_frame=samples_per_frame)
+        expected_size = fh.shape[0] - n + 1
         assert sh.sample_rate == fh.sample_rate
         assert sh.shape == (expected_size,) + fh.shape[1:]
         assert abs(sh.start_time
@@ -486,6 +504,16 @@ class TestPaddedTaskBase(UseVDIFSample):
         elif n == 1:
             expected = fh.read(10)
         data = sh.read(10)
+        # Also try from end.
+        if n == 3:
+            fh.seek(-12, 2)
+            raw = fh.read(12)
+            expected = raw[:-2] + raw[1:-1] + raw[2:]
+        elif n == 1:
+            fh.seek(-10, 2)
+            expected = fh.read(10)
+        sh.seek(-10, 2)
+        data = sh.read(10)
         assert np.all(data == expected)
         sh.close()
         assert sh.closed
