@@ -7,15 +7,15 @@ from numpy.testing import assert_array_equal
 import astropy.units as u
 from astropy.time import Time
 
-from ..base import Task
-from ..generators import EmptyStreamGenerator
-from ..integration import Integrate, Fold, Stack
-from ..functions import Square
-from ..phases import Phase
+from baseband_tasks.base import Task
+from baseband_tasks.generators import EmptyStreamGenerator
+from baseband_tasks.integration import Integrate, Fold, Stack
+from baseband_tasks.functions import Square
+from baseband_tasks.phases import Phase
 
 
-class TestFakePulsarBase:
-    def setup(self):
+class FakePulsarBase:
+    def setup_class(self):
         self.start_time = Time('2010-11-12T13:14:15')
         self.sample_rate = 10. * u.kHz
         self.shape = (16000, 2)
@@ -29,11 +29,8 @@ class TestFakePulsarBase:
         self.n_phase = 50
         self.raw_data = self.sh.read()
         self.raw_power = self.raw_data ** 2
-        self.sh.seek(0)
-
-    def phase(self, t):
-        return u.cycle * self.F0 * (t - self.start_time)
 
+    @classmethod
     def pulse_simulate(self, fh, data):
         idx = fh.tell() + np.arange(data.shape[0])
         result = np.where(idx % self.period_bin == 0, 10., 0.125)
@@ -41,13 +38,16 @@ class TestFakePulsarBase:
         data[:] = result
         return data
 
+    def phase(self, t):
+        return u.cycle * self.F0 * (t - self.start_time)
 
-class UsePhaseClass(TestFakePulsarBase):
+
+class UsePhaseClass(FakePulsarBase):
     def phase(self, t):
         return Phase(super().phase(t))
 
 
-class TestIntegrate(TestFakePulsarBase):
+class TestIntegrate(FakePulsarBase):
     """Test integrating intensities using Baseband's sample DADA file."""
 
     def test_integrate_all(self):
@@ -108,24 +108,30 @@ class TestIntegrate(TestFakePulsarBase):
         assert np.allclose(data, ref_data)
         assert np.all(count == self.sh.shape[0])
 
+    @pytest.mark.parametrize('seek', [121, -10])
     @pytest.mark.parametrize('samples_per_frame', (1, 4, 10))
     @pytest.mark.parametrize('n', (1, 3))
-    def test_integrate_n(self, n, samples_per_frame):
-        ref_data = self.raw_power[121 * n:131 * n].reshape(-1, n, 2).sum(1)
+    def test_integrate_n(self, n, samples_per_frame, seek):
+        n_sample = self.raw_power.shape[0] // n
+        seek = seek if seek > 0 else n_sample+seek
+        ref_data = (self.raw_power[seek * n:(seek+10) * n]
+                        .reshape(-1, n, 2).sum(1))
 
         st = Square(self.sh)
         ip = Integrate(st, n, average=False,
                        samples_per_frame=samples_per_frame)
+        assert ip.shape[0] == n_sample
         assert ip.start_time == self.sh.start_time
         assert ip.sample_rate == self.sh.sample_rate / n
 
-        ip.seek(121)
+        ip.seek(seek)
         assert abs(ip.time - (self.sh.start_time
-                              + 121 / ip.sample_rate)) < 1. * u.ns
+                              + seek / ip.sample_rate)) < 1. * u.ns
         integrated = ip.read(10)
         data = integrated['data']
         count = integrated['count']
-        assert ip.tell() == 131
+        assert ip.tell() == seek+10
+
         assert st.dtype is ref_data.dtype is data.dtype
         assert data.shape == ref_data.shape
         assert np.allclose(data, ref_data)
@@ -253,11 +259,9 @@ class TestIntegrate(TestFakePulsarBase):
             Integrate(self.sh, start=self.start_time+3.*u.s)
         with pytest.raises(AssertionError):
             Integrate(self.sh, step=1.*u.hr)
-        with pytest.raises(AssertionError):
-            Integrate(self.sh, samples_per_frame=2)
 
 
-class TestFold(TestFakePulsarBase):
+class TestFold(FakePulsarBase):
     def test_input_data(self):
         # Spot check on pulsar simulation.
         assert np.all(self.raw_data[::125] == 10.)
@@ -393,15 +397,13 @@ class TestFold(TestFakePulsarBase):
             Fold(self.sh, 8, self.phase, start=self.start_time + 3. * u.s)
         with pytest.raises(AssertionError):
             Fold(self.sh, 8, self.phase, step=1.*u.hr)
-        with pytest.raises(AssertionError):
-            Fold(self.sh, 8, self.phase, samples_per_frame=2)
 
 
 class TestFoldwithPhase(TestFold, UsePhaseClass):
     pass
 
 
-class TestIntegratePhase(TestFakePulsarBase):
+class TestIntegratePhase(FakePulsarBase):
     # More detailed tests done in TestStack.
     @pytest.mark.parametrize('samples_per_frame', (1, 160))
     def test_basics(self, samples_per_frame):
@@ -428,7 +430,7 @@ class TestIntegratePhasewithPhase(TestIntegratePhase, UsePhaseClass):
     pass
 
 
-class TestStack(TestFakePulsarBase):
+class TestStack(FakePulsarBase):
     @pytest.mark.parametrize('samples_per_frame', (1, 16))
     def test_basics(self, samples_per_frame):
         ref_data = self.raw_data.reshape(-1, 25, 5, 2).mean(2)
@@ -450,6 +452,16 @@ class TestStack(TestFakePulsarBase):
         data = fh.read()
         assert np.all(data == ref_data[13:])
 
+    @pytest.mark.parametrize('samples_per_frame', (1, 16))
+    def test_sliced_input(self, samples_per_frame):
+        ref_data = self.raw_data[-360:-110].reshape(-1, 25, 5, 2).mean(2)
+
+        fh = Stack(self.sh[-360:-10], 25, self.phase,
+                   samples_per_frame=samples_per_frame)
+        assert fh.shape == ref_data.shape
+        data = fh.read()
+        assert np.all(data == ref_data)
+
     def test_offset(self):
         ref_data = self.raw_data[124:-1].reshape(-1, 25, 5, 2).mean(2)
 
